"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGeneralValidateMiddleware = exports.extractAndValidate = exports.conditionalKeep = exports.extract = exports.generalFinalValidationCallbacks = exports.generalValidateCallbacks = void 0;
var utils_1 = require("./utils");
var validateUtils_1 = __importStar(require("./validateUtils"));
var _a = (0, validateUtils_1.default)(createGeneralValidation, createGeneralValidateMiddleware), stringValidate = _a.stringValidate, numberValidate = _a.numberValidate;
exports.generalValidateCallbacks = {
    maxLength: stringValidate.maxLength,
    exactLength: stringValidate.exactLength,
};
exports.generalFinalValidationCallbacks = {
    greaterOrEqualTo: numberValidate.greaterOrEqualTo,
};
function basicVisitWithFollowerVerification(variableName, type, variableSettings, follower, valueArrayCb) {
    function constructError(value, variableName, follower, s) {
        return "The value \"" + value + "\" that belongs to the key \"" + variableName + "\" in the object \"" + JSON.stringify(follower) + "\" " + s;
    }
    if (follower === undefined)
        throw new Error("Please supply an object to be validated");
    var value = follower[variableName];
    var isCorrectType = function (v) {
        try {
            return typeof v === variableSettings.type || v instanceof type;
        }
        catch (_a) {
            return false;
        }
    };
    var valueArray;
    if (variableSettings.array === true) {
        if (!Array.isArray(value))
            throw new Error(constructError(value, variableName, follower, "is not an array"));
        valueArray = value;
    }
    else
        valueArray = [value];
    valueArray.forEach(function (value) {
        // do more processing to check if it works
        if (!isCorrectType(value))
            throw new Error(constructError(value, variableName, follower, "is not of the expected type \"" + variableSettings.type + "\""));
        valueArrayCb === null || valueArrayCb === void 0 ? void 0 : valueArrayCb(value);
    });
    return value;
}
function extract(follower, rules) {
    var flattened = (0, utils_1.flattenIfNeeded)(rules);
    return (0, utils_1.visit)(flattened, function (variableName, toModify, type, variableSettings, follower) {
        var value = basicVisitWithFollowerVerification(variableName, type, variableSettings, follower);
        toModify[variableName] = value;
    }, function (variableName, toModify, childObj) {
        toModify[variableName] = childObj;
    }, utils_1.fieldTypesByTypeName, follower);
}
exports.extract = extract;
function conditionalKeep(follower, rules, key, predicate) {
    var flattened = (0, utils_1.flattenIfNeeded)(rules);
    return (0, utils_1.visit)(flattened, function (variableName, toModify, type, variableSettings, follower) {
        var value = basicVisitWithFollowerVerification(variableName, type, variableSettings, follower);
        if (key in variableSettings && !predicate(variableSettings[key]))
            return;
        toModify[variableName] = value;
    }, function (variableName, toModify, childObj, variableSettings) {
        if (key in variableSettings && !predicate(variableSettings[key]))
            return;
        toModify[variableName] = childObj;
    }, utils_1.fieldTypesByTypeName, follower);
}
exports.conditionalKeep = conditionalKeep;
function extractAndValidate(follower, rules) {
    var flattened = (0, utils_1.flattenIfNeeded)(rules); // make sure that all the needed keys are there
    var finalValidationCalls = [];
    var out = (0, utils_1.visit)(flattened, function (variableName, toModify, type, variableSettings, follower) {
        var value = basicVisitWithFollowerVerification(variableName, type, variableSettings, follower, function (v) {
            if (variableSettings.enum !== undefined) {
                if (!variableSettings.enum.includes(v))
                    throw new Error("The enum \"" + variableName + "\" with the value \"" + follower[variableName] + "\" must have one of the following values: \"" + variableSettings.enum + "\"");
            }
            Object.entries(exports.generalValidateCallbacks).forEach(function (_a) {
                var cbName = _a[0], cb = _a[1];
                if (variableSettings[cbName] !== undefined) {
                    try {
                        cb(v, variableSettings[cbName]);
                    }
                    catch (e) {
                        throw new Error("The \"" + cbName + "\" constraint with the value \"" + variableSettings[cbName] + "\" was not met by the field \"" + variableName + "\" with the value \"" + v + "\" with an error message saying \"" + e.message + "\"");
                    }
                }
            });
            Object.entries(exports.generalFinalValidationCallbacks).forEach(function (_a) {
                var cbName = _a[0], cb = _a[1];
                if (variableSettings[cbName] !== undefined) {
                    var options = variableSettings[cbName];
                    // the first value is the validation callback, the others are the values to check
                    var outVals = [cb, variableName];
                    if (Array.isArray(options))
                        outVals.push.apply(outVals, options);
                    // TODO: test this
                    else
                        outVals.push(options);
                    finalValidationCalls.push(outVals);
                }
            });
        });
        toModify[variableName] = value;
    }, function (variableName, toModify, childObj) {
        toModify[variableName] = childObj;
    }, utils_1.fieldTypesByTypeName, follower);
    Object.entries(finalValidationCalls).forEach(function (_a) {
        var callName = _a[0], callOptions = _a[1];
        var validator = callOptions.shift();
        try {
            validator(callOptions, follower);
        }
        catch (e) {
            throw new Error("A constraint failed with an error message saying \"" + e.message + "\"");
        }
    });
    return out;
}
exports.extractAndValidate = extractAndValidate;
function createGeneralValidation(message, validate) {
    return function (value) {
        var checkSettings = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            checkSettings[_i - 1] = arguments[_i];
        }
        // options specific to the current value
        var result = validate.apply(null, __spreadArray([value], checkSettings, true));
        if (!result)
            throw new Error(message);
        return result;
    };
}
function createGeneralValidateMiddleware(message, automaticResponseToUndefined, callback) {
    return function (fieldsRequiredToValidate, toValidate) {
        if ((0, validateUtils_1.callOverallValidationFunction)(fieldsRequiredToValidate, toValidate, automaticResponseToUndefined, callback))
            return true;
        throw new Error(message + "; Fields for validation: " + fieldsRequiredToValidate);
    };
}
exports.createGeneralValidateMiddleware = createGeneralValidateMiddleware;
//# sourceMappingURL=general.js.map