"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mongoCreateValidateMiddleware = exports.mongoCreateValidation = exports.constructSchema = exports.mongoFinalValidationCallbacks = exports.mongoValidateCallbacks = exports.mongoValuesToApply = void 0;
var mongoose_1 = require("mongoose");
var validateUtils_1 = __importStar(require("./validateUtils"));
var utils_1 = require("./utils");
var _a = (0, validateUtils_1.default)(mongoCreateValidation, mongoCreateValidateMiddleware), stringValidate = _a.stringValidate, numberValidate = _a.numberValidate;
exports.mongoValuesToApply = ["default", "enum", "unique"];
exports.mongoValidateCallbacks = {
    maxLength: stringValidate.maxLength,
    exactLength: stringValidate.exactLength
};
exports.mongoFinalValidationCallbacks = {
    greaterOrEqualTo: numberValidate.greaterOrEqualTo
};
function constructSchema(inputFields) {
    var flattened = (0, utils_1.flattenIfNeeded)(inputFields);
    var finalValidationCallbacksForThisSchema = [];
    var schemaDef = (0, utils_1.visit)(flattened, function (variableName, out, type, variableSettings) {
        var target = out[variableName] = {};
        target.required = variableSettings.required;
        target.type = variableSettings.array === true ? [type] : type;
        exports.mongoValuesToApply.forEach(function (name) { if (variableSettings[name] !== undefined)
            target[name] = variableSettings[name]; });
        Object.entries(exports.mongoValidateCallbacks).forEach(function (_a) {
            var cbName = _a[0], cb = _a[1];
            if (variableSettings[cbName] !== undefined)
                target.validate = cb(variableSettings[cbName]);
        });
        Object.entries(exports.mongoFinalValidationCallbacks).forEach(function (_a) {
            var cbName = _a[0], cb = _a[1];
            if (variableSettings[cbName] === undefined)
                return;
            // the first value is the validation callback, the others are the values to check (starting with self)
            var outVals = [cb, variableName];
            var options = variableSettings[cbName];
            if (Array.isArray(options))
                outVals.push.apply(outVals, options);
            else
                outVals.push(options);
            finalValidationCallbacksForThisSchema.push(outVals);
        });
    }, function (variableName, out, childObj, variableSettings) {
        var target = out[variableName] = {};
        target.required = variableSettings.required;
        target.type = variableSettings.array === true ? [new mongoose_1.Schema(childObj)] : new mongoose_1.Schema(childObj);
    }, utils_1.fieldTypesByTypeName);
    var out = new mongoose_1.Schema(schemaDef);
    finalValidationCallbacksForThisSchema.forEach(function (settings) {
        var validator = settings.shift();
        out.pre("validate", validator(settings));
    });
    return out;
}
exports.constructSchema = constructSchema;
// "current" means that those values are no longer general
function mongoCreateValidation(message, validate) {
    return function () {
        var checkSettings = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            checkSettings[_i] = arguments[_i];
        }
        return {
            message: message,
            validator: function (value) { return validate.apply(null, __spreadArray([value], checkSettings, true)); }
        };
    };
}
exports.mongoCreateValidation = mongoCreateValidation;
function mongoCreateValidateMiddleware(message, automaticResponseToUndefined, callback) {
    return function (fieldsRequiredToValidate) {
        var out = function (next) {
            if ((0, validateUtils_1.callOverallValidationFunction)(fieldsRequiredToValidate, this, automaticResponseToUndefined, callback))
                return next();
            throw new Error(message + "; Fields for validation: " + fieldsRequiredToValidate);
        };
        return out;
    };
}
exports.mongoCreateValidateMiddleware = mongoCreateValidateMiddleware;
//# sourceMappingURL=mongo.js.map