"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fieldTypesByTypeName = exports.deleteEmptyAndUndefined = exports.visit = exports.flattenIfNeeded = exports.flatten = exports.deepAssign = void 0;
//NOTE: this function is NOT SAFE and therefore should never be used with user input
function deepAssign(target, src) {
    if (src === undefined)
        return target; // return target if there is nothing in src to override target with
    var isRecursible = function (val) { return typeof val === "object" && val !== null; };
    if (!isRecursible(target) || !isRecursible(src))
        return src; // if we can not recurse anymore, then choose src
    var keys = new Set(Object.keys(target));
    Object.keys(src).forEach(function (k) { return keys.add(k); }); // a set allows us not to bother about duplicates
    var out = (Array.isArray(target)) ? [] : {}; // the original Object.assign seems to select the type based on the target
    keys.forEach(function (k) {
        if (src.hasOwnProperty(k) && src[k] === undefined)
            return; // if the property was specifically overwritten, leave it out
        out[k] = deepAssign(target[k], src[k]);
    });
    return out;
}
exports.deepAssign = deepAssign;
//NOTE: this function is NOT SAFE and therefore should never be used with user input
function flatten(fields) {
    var out = {};
    Object.entries(fields).forEach(function (_a) {
        var typeContainerKey = _a[0], typeContainer = _a[1];
        // if it is an additional setting of an object, do not touch it because it has been recorded already. NOTE: the type before this point is incorrect because the supplied value could be AdditionalObjectFieldConstraintsCollection 
        if (!["required", "optional"].includes(typeContainerKey))
            return;
        var isRequired = typeContainerKey === "required";
        Object.entries(typeContainer).forEach(function (_a) {
            var groupName = _a[0], fieldGroup = _a[1];
            Object.entries(fieldGroup).forEach(function (_a) {
                var variableName = _a[0], variableSettings = _a[1];
                // recurse if needed
                var variableSettingsOut = {};
                if (groupName === "object") {
                    var fieldConstraintsCollection = variableSettings;
                    // add all the other keys, e.g. "array" if needed
                    variableSettingsOut = Object.assign({}, fieldConstraintsCollection);
                    delete variableSettingsOut.required;
                    delete variableSettingsOut.optional; // removing those entries so that it becomes an AdditionalObjectFieldConstraintsCollection
                    out[variableName] = {
                        type: flatten(fieldConstraintsCollection),
                        "required": isRequired,
                    }; // e.g. type: {Object}, required: true
                }
                else {
                    // else, copy over all the needed values
                    out[variableName] = {
                        type: groupName,
                        required: isRequired,
                    };
                    variableSettingsOut = variableSettings;
                }
                out[variableName] = __assign(__assign({}, out[variableName]), variableSettingsOut);
            });
        });
    });
    return out;
}
exports.flatten = flatten;
function flattenIfNeeded(input) {
    function isFlattened(v) {
        return Object.entries(v).every(function (_a) {
            var k = _a[0], v = _a[1];
            return "type" in v;
        });
    }
    return isFlattened(input) ? input : flatten(input);
}
exports.flattenIfNeeded = flattenIfNeeded;
function tryFollow(obj, key) {
    try {
        return obj[key];
    }
    catch (_a) {
        return undefined;
    }
}
function visit(fields, resolvePrimitive, addObject, typeMap, follower) {
    var out = {};
    Object.entries(fields).forEach(function (_a) {
        var variableName = _a[0], variableSettings = _a[1];
        // const followerVariable = tryFollow(follower, variableName);
        var type = variableSettings.type;
        // if it is not an object or an array, make sure that it can't be passed down because everything will break
        if (follower !== undefined && (follower === null || !(Array.isArray(follower) || Object.getPrototypeOf(follower).isPrototypeOf(Object))))
            throw new Error("The supplied value " + JSON.stringify(follower) + " should be an object or an array, but it is not");
        // if the key is not present or can not be extracted
        if (follower !== undefined && tryFollow(follower, variableName) === undefined) {
            if (variableSettings.required)
                throw new Error("The passed value \"" + JSON.stringify(follower) + "\" does not have the required key \"" + variableName + "\" as requested by the schema \"" + JSON.stringify(fields) + "\"");
            else
                return;
        }
        if (typeof type === "string") { // if it is a primitive
            var convertedType = typeMap ? typeMap[type] : type;
            resolvePrimitive(variableName, out, convertedType, variableSettings, follower); // modifies "out"
        }
        else if (typeof type === "object") {
            var followerValue = tryFollow(follower, variableName);
            var nested = void 0;
            if (follower !== undefined && variableSettings.array === true) {
                if (!Array.isArray(followerValue))
                    throw new Error("The passed value \"" + JSON.stringify(followerValue) + "\" corresponding to the key " + variableName + " was expected to be an array as requested by the schema \"" + JSON.stringify(fields) + "\"");
                nested = followerValue.map(function (followerEl) { return visit(type, resolvePrimitive, addObject, typeMap, followerEl); });
            }
            else
                nested = visit(type, resolvePrimitive, addObject, typeMap, followerValue);
            addObject(variableName, out, nested, variableSettings);
        }
    });
    return out;
}
exports.visit = visit;
// NOTE: also deletes undefined
function deleteEmptyAndUndefined(target) {
    // if not a plain object, skip
    if (target === null || target === undefined || !Object.getPrototypeOf(target).isPrototypeOf(Object))
        return target;
    if (Object.keys(target).length === 0)
        return undefined;
    var out = {};
    Object.entries(target).forEach(function (_a) {
        var objKey = _a[0], objEntry = _a[1];
        // if an empty object, do not add
        var newVal = deleteEmptyAndUndefined(objEntry);
        if (newVal !== undefined)
            out[objKey] = newVal;
    });
    return out;
}
exports.deleteEmptyAndUndefined = deleteEmptyAndUndefined;
exports.fieldTypesByTypeName = {
    string: String,
    number: Number,
    boolean: Boolean,
    date: Date,
    object: Object,
};
//# sourceMappingURL=utils.js.map