"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.callOverallValidationFunction = exports.isMaxLength = exports.isExactLength = exports.isGreaterOrEqualTo = void 0;
function isGreaterOrEqualTo(resolvedFields) {
    return resolvedFields.slice(1).every(function (v) { return resolvedFields[0] >= v; });
}
exports.isGreaterOrEqualTo = isGreaterOrEqualTo;
var isExactLength = function (value, length) { return value.length == length; };
exports.isExactLength = isExactLength;
var isMaxLength = function (value, length) { return value.length <= length; };
exports.isMaxLength = isMaxLength;
function injectValidationCreators(createValidation, createFinalValidation) {
    return {
        stringValidate: {
            exactLength: createValidation("This string field size does not match the required size", exports.isExactLength),
            maxLength: createValidation("This string field is too long", exports.isMaxLength),
        },
        numberValidate: {
            greaterOrEqualTo: createFinalValidation("The fields need to be in descending order or equal", false, isGreaterOrEqualTo)
        }
    };
}
exports.default = injectValidationCreators;
function callOverallValidationFunction(fieldsRequiredToValidate, toValidate, automaticResponseToUndefined, callback) {
    // the first one is the value, the others are the arguments
    var resolvedRawvalues = toValidate[fieldsRequiredToValidate[0]];
    var values = Array.isArray(resolvedRawvalues) ? resolvedRawvalues : [resolvedRawvalues];
    var unresolvedArgs = fieldsRequiredToValidate.slice(1);
    var resolvedArgs = [];
    unresolvedArgs.forEach(function (v) {
        var out = toValidate[v];
        if (Array.isArray(out))
            resolvedArgs.push.apply(resolvedArgs, out); // TODO: test this separately
        else
            resolvedArgs.push(out);
    });
    return values.every(function (v, i) {
        var resolvedInputArray = [v].concat(resolvedArgs);
        return automaticResponseToUndefined !== null && resolvedInputArray.some(function (k) { return k === undefined; }) ? // if has undefined fields and we need to care about them,
            automaticResponseToUndefined : // return the pre-defined value
            callback(resolvedInputArray, toValidate, [fieldsRequiredToValidate[0]].concat(unresolvedArgs)); // else, validate
    });
}
exports.callOverallValidationFunction = callOverallValidationFunction;
//# sourceMappingURL=validateUtils.js.map